"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZiGateMessage = void 0;
/* istanbul ignore file */
/* eslint-disable */
const constants_1 = require("./constants");
exports.ZiGateMessage = {
    [constants_1.ZiGateMessageCode.GetTimeServer]: {
        response: [
            { name: 'timestampUTC', parameterType: 'UINT32' },
        ]
    },
    [constants_1.ZiGateMessageCode.DeviceAnnounce]: {
        response: [
            { name: 'shortAddress', parameterType: 'UINT16BE' },
            { name: 'ieee', parameterType: 'IEEEADDR' },
            { name: 'MACcapability', parameterType: 'MACCAPABILITY' },
            // MAC capability
            // Bit 0 – Alternate PAN Coordinator
            // Bit 1 – Device Type
            // Bit 2 – Power source
            // Bit 3 – Receiver On when Idle
            // Bit 4,5 – Reserved
            // Bit 6 – Security capability
            // Bit 7 – Allocate Address
            { name: 'rejoin', parameterType: 'UINT8' },
        ]
    },
    [constants_1.ZiGateMessageCode.Status]: {
        response: [
            { name: 'status', parameterType: 'UINT8' },
            // 0 = Success
            // 1 = Incorrect parameters
            // 2 = Unhandled command
            // 3 = Command failed
            // eslint-disable-next-line max-len
            // 4 = Busy (Node is carrying out a lengthy operation and is currently unable to handle the incoming command)
            // 5 = Stack already started (no new configuration accepted)
            // 128 – 244 = Failed (ZigBee event codes)
            // Packet Type: The value of the initiating command request.
            { name: 'sequence', parameterType: 'UINT8' },
            { name: 'packetType', parameterType: 'UINT16BE' },
            { name: 'requestSent', parameterType: 'UINT8' },
            // a device(aps ack/nack 8011 should be expected) , 0 otherwise
            { name: 'seqApsNum', parameterType: 'UINT8' },
        ]
    },
    [constants_1.ZiGateMessageCode.PermitJoinStatus]: {
        response: [
            { name: 'status', parameterType: 'UINT8' },
        ]
    },
    [constants_1.ZiGateMessageCode.DataIndication]: {
        response: [
            { name: 'status', parameterType: 'UINT8' },
            { name: 'profileID', parameterType: 'UINT16BE' },
            { name: 'clusterID', parameterType: 'UINT16BE' },
            { name: 'sourceEndpoint', parameterType: 'UINT8' },
            { name: 'destinationEndpoint', parameterType: 'UINT8' },
            { name: 'sourceAddressMode', parameterType: 'UINT8' },
            { name: 'sourceAddress', parameterType: 'ADDRESS_WITH_TYPE_DEPENDENCY' },
            // <source address: uint16_t or uint64_t>
            { name: 'destinationAddressMode', parameterType: 'UINT8' },
            // <destination address mode: uint8_t>
            { name: 'destinationAddress', parameterType: 'ADDRESS_WITH_TYPE_DEPENDENCY' },
            // <destination address: uint16_t or uint64_t>
            // {name: 'payloadSize', parameterType:'UINT8'}, // <payload size : uint8_t>
            { name: 'payload', parameterType: 'BUFFER_RAW' },
        ]
    },
    [constants_1.ZiGateMessageCode.APSDataConfirm]: {
        response: [
            { name: 'status', parameterType: 'UINT8' },
            // {name: 'sourceEndpoint', parameterType:'UINT8'}, // <source endpoint: uint8_t>
            // {name: 'destinationAddressMode', parameterType:'UINT8'},
            // // <destination address mode: uint8_t>
            { name: 'destinationAddress', parameterType: 'UINT16BE' },
            { name: 'destinationEndpoint', parameterType: 'UINT8' },
            { name: 'clusterID', parameterType: 'UINT16BE' },
            // // <destination address: uint16_t or uint64_t>
            { name: 'seqNumber', parameterType: 'UINT8' },
        ]
    },
    [constants_1.ZiGateMessageCode.NetworkState]: {
        response: [
            { name: 'shortAddress', parameterType: 'UINT16BE' },
            { name: 'extendedAddress', parameterType: 'IEEEADDR' },
            { name: 'PANID', parameterType: 'UINT16BE' },
            { name: 'ExtPANID', parameterType: 'IEEEADDR' },
            { name: 'Channel', parameterType: 'UINT8' },
        ]
    },
    [constants_1.ZiGateMessageCode.VersionList]: {
        response: [
            { name: 'majorVersion', parameterType: 'UINT16BE' },
            { name: 'installerVersion', parameterType: 'UINT16BE' },
        ]
    },
    [constants_1.ZiGateMessageCode.NetworkJoined]: {
        response: [
            { name: 'status', parameterType: 'UINT8' },
            // Status:
            // 0 = Joined existing network
            // 1 = Formed new network
            // 128 – 244 = Failed (ZigBee event codes)
            { name: 'shortAddress', parameterType: 'UINT16BE' },
            { name: 'extendedAddress', parameterType: 'IEEEADDR' },
            { name: 'channel', parameterType: 'UINT8' },
        ]
    },
    [constants_1.ZiGateMessageCode.LeaveIndication]: {
        response: [
            { name: 'extendedAddress', parameterType: 'IEEEADDR' },
            { name: 'rejoin', parameterType: 'UINT8' },
        ]
    },
    [constants_1.ZiGateMessageCode.RouterDiscoveryConfirm]: {
        response: [
            { name: 'status', parameterType: 'UINT8' },
            { name: 'nwkStatus', parameterType: 'UINT8' },
        ]
    },
    [constants_1.ZiGateMessageCode.APSDataConfirmFail]: {
        response: [
            { name: 'status', parameterType: 'UINT8' },
            { name: 'sourceEndpoint', parameterType: 'UINT8' },
            { name: 'destinationEndpoint', parameterType: 'UINT8' },
            { name: 'destinationAddressMode', parameterType: 'UINT8' },
            { name: 'destinationAddress', parameterType: 'ADDRESS_WITH_TYPE_DEPENDENCY' },
            // <destination address: uint64_t>
            { name: 'seqNumber', parameterType: 'UINT8' },
        ]
    },
    [constants_1.ZiGateMessageCode.ActiveEndpointResponse]: {
        response: [
            { name: 'sequence', parameterType: 'UINT8' },
            { name: 'status', parameterType: 'UINT8' },
            { name: 'nwkAddr', parameterType: 'UINT16BE' },
            { name: 'endpointCount', parameterType: 'UINT8' },
            { name: 'endpoints', parameterType: 'LIST_UINT8' },
        ]
    },
    // [ZiGateMessageCode.SimpleDescriptorResponse]: {
    //     response: [
    //         {name: 'sourceEndpoint', parameterType:'UINT8'}, //<source endpoint: uint8_t>
    //         {name: 'profile ID', parameterType:'UINT16BE'}, // <profile ID: uint16_t>
    //         {name: 'clusterID', parameterType:'UINT16BE'}, // <cluster ID: uint16_t>
    //         {name: 'attributeList', parameterType:'LIST_UINT16BE'}, // <attribute list: data each entry is uint16_t>
    //     ]
    // },
    [constants_1.ZiGateMessageCode.ManagementLQIResponse]: {
        response: [
            { name: 'sequence', parameterType: 'UINT8' },
            { name: 'status', parameterType: 'UINT8' },
            { name: 'neighbourTableEntries', parameterType: 'UINT8' },
            { name: 'neighbourTableListCount', parameterType: 'UINT8' },
            { name: 'startIndex', parameterType: 'UINT8' },
            // @TODO list TYPE
            // <List of Entries elements described below :>
            // Note: If Neighbour Table list count is 0, there are no elements in the list.
            { name: 'NWKAddress', parameterType: 'UINT16BE' },
            { name: 'Extended PAN ID', parameterType: 'UINT64' },
            { name: 'IEEE Address', parameterType: 'IEEEADR' },
            { name: 'Depth', parameterType: 'UINT8' },
            { name: 'linkQuality', parameterType: 'UINT8' },
            { name: 'bitMap', parameterType: 'UINT8' },
            // bit 0-1 Device Type
            // (0-Coordinator 1-Router 2-End Device)
            // bit 2-3 Permit Join status
            // (1- On 0-Off)
            // bit 4-5 Relationship
            // (0-Parent 1-Child 2-Sibling)
            // bit 6-7 Rx On When Idle status
            // (1-On 0-Off)
            { name: 'srcAddress', parameterType: 'UINT16BE' },
        ]
    },
    [constants_1.ZiGateMessageCode.PDMEvent]: {
        response: [
            { name: 'eventStatus', parameterType: 'UINT8' },
            { name: 'recordID', parameterType: 'UINT32BE' },
        ]
    },
    [constants_1.ZiGateMessageCode.RestartNonFactoryNew]: {
        response: [
            { name: 'status', parameterType: 'UINT8' },
        ]
    },
    [constants_1.ZiGateMessageCode.RestartFactoryNew]: {
        response: [
            { name: 'status', parameterType: 'UINT8' },
        ]
    }
};
//# sourceMappingURL=messageType.js.map